from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import tempfile

import pytest

from s3_uploader.exceptions.validation_exception import ValidationException
from s3_uploader.uploader.validation import validate_file


def test_validate_file_is_dir():
    """A test to check the functionality of the validate_file function.
    A path to a directory is passed to the function, and so we expect to get a 
    ValidationException exception to be raised as this is not an accessible 
    file path.
     
    :return: Nothing. A ValidationException should be raised. 
    """
    with pytest.raises(ValidationException):
        path = '/tmp/'
        validate_file(path)


def test_validate_file_is_not_accessible():
    """A test to check the functionality of the validate_file function.
    A path to a non-accessible file is passed to the function, and so we 
    expect to get a ValidationException exception to be raised as this is not 
    an accessible file path.

    :return: Nothing. A ValidationException should be raised. 
    """
    with pytest.raises(ValidationException):
        path = '/root/some_file'
        validate_file(path)


def test_validate_file_good():
    """A test to check the functionality of the validate_file function.
    A path to an accessible file is passed to the function, and so we 
    expect the function to return True as the path to the file is accessible.

    :return: True, unless the function is not working as we expected. 
    """
    f = tempfile.NamedTemporaryFile()
    is_validate = validate_file(f.name)
    f.close()
    assert is_validate is True



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os

from s3_uploader.utils.utils import get_real_file_path


def test_get_real_file_path_absolute():
    """A test to check the functionality of the get_real_file_path function.
    An absolute path is being passed to the function, and so we expect the 
    path to stay as is.
    
    :return: True, unless the function is not working as we expected.
    """
    absolute_path = '/tmp'
    path = get_real_file_path(absolute_path)
    assert path == absolute_path


def test_get_real_file_path_relative():
    """A test to check the functionality of the get_real_file_path function.
    A relative path is being passed to the function, and so we expect the 
    path to be returned as absolute path in respect to the current working 
    directory.
    
    :return: True, unless the function is not working as we expected.
    """
    relative_path = 'path/to/file'
    path = get_real_file_path(relative_path)
    expected_path = os.path.join(os.getcwd(), relative_path)
    assert path == expected_path



from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import traceback

import sys


def handle_exception(msg=''):
    """This function is the project's exception handler.
    There is currently no logic, but could be easily added in the future.
    
    :param msg: The message to print to the user. 
    :return: Nothing, actually exits the program.
    """
    print(msg)
    traceback.print_exc()
    sys.exit(1)



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os

from s3_uploader.exceptions.validation_exception import ValidationException
from s3_uploader.utils.utils import get_real_file_path


def validate_file(file_to_upload):
    """Validating the file path.

    Validation rules are:
    1. Path should contain a file at the end and not point to a directory.
    2. File should be accessible so it can be uploaded.

    In case the path is relative, we'll use the OS current working directory
    and join its path with the given file path.

    In case of any violation of the validation process,
    an exception will be raised.

    :param file_to_upload: The path to the file. Can be either relative or
    absolute path.
    
    :return: True if everything goes by the rules, a ValidationException 
    otherwise.
    """
    upload_file_path = get_real_file_path(file_to_upload)

    if os.path.isdir(upload_file_path):
        raise ValidationException('Path should contain file at the end')

    # Verifying that the file is accessible before uploading
    try:
        with open(upload_file_path, 'rb'):
            pass
    except IOError:
        raise ValidationException('The file could not be opened. Check path '
                                  'and/or permissions')
    return True



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import sys

from s3_uploader.exceptions.validation_exception import ValidationException
from s3_uploader.uploader.upload import upload_resource_and_return_version
from s3_uploader.uploader.validation import validate_file
from s3_uploader.utils.exception_handler import handle_exception
from s3_uploader.utils.utils import generate_json_output, print_output_message


def handle_resource_upload(args):
    """Main function to upload the resource to S3.

    The resource will be uploaded to the research binaries S3 repository,
    and a JSON output will be returned to the user to add to the microservice
    dependencies.json configuration file.

    :param args: The user's arguments supplied from the main function.
    """
    try:
        validate_file(args.file)
        version, s3_path = upload_resource_and_return_version(
            file_to_upload=args.file,
            bucket=args.bucket,
            bucket_path=args.s3_path)

        json_response = generate_json_output(
            args.location, version, s3_path)

        print_output_message(json_response)
        sys.exit()

    except ValidationException as ex:
        handle_exception(ex.message)

    except Exception:
        handle_exception('Unknown error occurred')



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
from datetime import datetime

from s3_uploader.utils import s3_utils
from s3_uploader.utils.utils import get_real_file_path


def upload_resource_and_return_version(file_to_upload, bucket,
                                       bucket_path=None):
    """This function responsible for uploading and versioning the resource.

    :param file_to_upload: A path to file to be uploaded.
    :param bucket: The name of the bucket.
    :param bucket_path: path to file in S3(without bucket name)
    :return: The version for the recent uploaded file.
    """
    file_name = os.path.basename(file_to_upload)
    version = datetime.today().strftime("%Y-%m-%d-%H-%M-%S")

    if not bucket_path:
        bucket_path = '/'.join([file_name, version, file_name])

    file_path = get_real_file_path(file_to_upload)
    s3_utils.upload(file_path, bucket, bucket_path)

    return version, bucket_path





from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import json
import os
from collections import OrderedDict


def get_real_file_path(file_path):
    """This function gets a file path, can be either relative or absolute, 
    parse it and return the absolute real file path.

    If the file path contains tilde, it will be expanded to the user's home
    directory if set.
    If the file path is relative, it will be expanded in respect to the
    current working directory.

    :param file_path: A path to a file.
    :return: The absolute real path for the file.
    """
    file_path = os.path.expanduser(file_path)
    file_path = os.path.abspath(file_path)
    return file_path


def generate_json_output(future_location, version, s3_path):
    """This function generates a JSON output response for placing in the
    project's dependencies configuration file.

    :param future_location: The location in which the file should be placed
    in the user's project.
    :param version: The version generated by the tool for this file.
    :param s3_path: path to the file location in S3

    :return: The JSON response with the fields ordered by the creation order.
    """
    json_response = OrderedDict()
    json_response['name'] = os.path.basename(s3_path)
    json_response['version'] = version
    json_response['location'] = future_location
    json_response['s3path'] = s3_path

    return json_response


def print_output_message(json_response):
    """This function prints the JSON response along with some more information
    on what to do with the response.

    :param json_response: The JSON response to print. Should be a dict-like
    object.

    :return: Nothing, just prints to stdout. 
    """
    msg = 'The file {0} has been successfully uploaded!\nAdd this json to ' \
          'your microservice dependencies.json ' \
          'configuration file, as another item in the "dependencies" ' \
          'list:\n\n{1}\n'.format(json_response['name'],
                                  json.dumps(json_response, indent=2))
    print(msg)



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import boto3
from boto3.exceptions import S3UploadFailedError
from botocore.exceptions import ClientError

from s3_uploader.utils.exception_handler import handle_exception

# Credentials are being read from shared credentials file configured for
# the AWS command line. Usually this configured as: ~/.aws/credentials

# More details at:
# http://boto3.readthedocs.io/en/latest/guide/configuration.html#shared-credentials-file
s3_client = boto3.client('s3')


def upload(file_path, bucket, bucket_path):
    """This function wraps boto3's s3 upload function, accompanied with a 
    try/except mechanism for exception handling.
    
    At the moment, the exception handling is basic and practically doesn't 
    do much, but in the future can be extended.
    
    :param file_path: The path to the file to upload. 
    :param bucket: The name of the bucket.
    :param bucket_path: The key path inside the bucket for the file to be 
    stored at.
    :return: Nothing.
    """
    try:
        s3_client.upload_file(file_path, bucket, bucket_path)
    except (ClientError, ValueError, S3UploadFailedError) as ex:
        handle_exception(ex.message)



from __future__ import print_function
from __future__ import division

from setuptools import setup, find_packages
from pip.req import parse_requirements

with open('README.rst') as f:
    readme = f.read()

with open('LICENSE') as f:
    _license = f.read()

install_requirements = parse_requirements('requirements.txt', session=False)
requirements = [str(ir.req) for ir in install_requirements]

setup(
    name='s3-uploader',
    version='0.0.5',
    description='Command line tool for uploading resources to S3',
    long_description=readme,
    author='Onfido',
    url='https://github.com/onfido/s3-uploader/',
    license='MIT',
    packages=find_packages(exclude=('tests', 'docs')),
    include_package_data=True,
    install_requires=requirements,
    entry_points={
        'console_scripts': [
            's3-uploader = s3_uploader.main:main', ], },
    keywords=['python', 's3', 'aws', 'upload', 'binaries', 'resources',
              'boto3', 'uploader']
)



from __future__ import division
from __future__ import print_function

import argparse

from uploader.handle_resource import handle_resource_upload


def create_parser():
    """A function to create the argument parser to parse the user's input.
    
    :return: The argument parser.
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('-f', '--file', dest='file',
                        required=True, action='store',
                        help='The file path to be uploaded. Can either be '
                             'relative or absolute path')
    parser.add_argument('-l', '--location', dest='location',
                        required=True, action='store',
                        help='The location in which the file should be '
                             'placed in the microservice. This location '
                             'should be a relative path regarding the '
                             'microservice root folder. This is for '
                             'generating the JSON to add '
                             'to the dependencies configuration file')
    parser.add_argument('-b', '--bucket', dest='bucket',
                        required=True, action='store',
                        help='The name of the bucket to use.')
    parser.add_argument('-p', '--s3-path', dest='s3_path',
                        action='store',
                        help='Path to the file in S3')
    return parser


def parse_arguments():
    """A function to parse the user's arguments and return an object which 
    holds the user's parsed arguments.
    
    :return: The user's parsed arguments as object.
    """
    parser = create_parser()
    args = parser.parse_args()
    return args


def main():
    """The main function to handle the user's input."""
    arguments = parse_arguments()
    handle_resource_upload(arguments)


if __name__ == '__main__':
    main()
